--[[
 MiniPet Addon
 Version 3.0
 $Revision: 75305 $

 Currently maintained by Breanni of Scarlet Crusade
 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage
    
    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning
    
    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).
    
    Updated and converted to Ace2 by LordFarlander
]]--

local L = AceLibrary( "AceLocale-2.2" ):new( "MiniPet" );
local LBZ = AceLibrary( "LibBabble-Zone-3.0" ):GetLookupTable();

_G["BINDING_NAME_CLICK MiniPet_Button:LeftButton"] = L["Summon MiniPet"];
BINDING_HEADER_MINIPETHEADER = L["MiniPet"];

local options = {
        type = "group",
        args = {
            [L["toggleAutoHideButton"]] = {
                type = "toggle",
                name = L["Auto Hide Button"],
                desc = L["Hide the button when you clicked it?"],
                get = function()
                          return MiniPet.db.char.AutoHideButton;
                      end,
                set = function( val )
                          MiniPet.db.char.AutoHideButton = not MiniPet.db.char.AutoHideButton;
                      end,
            },
--         toggleDebug = {
--             type = "toggle",
--             name = "Debug",
--             desc = "Debug mode toggle",
--             get = function()
--                       return MiniPet.DEBUG;
--                   end,
--             set = function( val )
--                       MiniPet.DEBUG = not self.DEBUG;
--                   end,
--         },
            [L["toggleButton"]] = {
                type = "toggle",
                name = L["Show Button"],
                desc = L["Show/Hide the MiniPet-Button."],
                get = function()
                          return MiniPet_Button:IsVisible()
                      end,
                set = function( val )
                          if( MiniPet_Button:IsVisible() ) then
                              MiniPet_Button:Hide();
                          else
                              MiniPet_Button:Show();
                          end
                      end,
            },
            [L["toggleTextWarning"]] = {
                type = "toggle",
                name = L["Text Warning"],
                desc = L["Show text warnings to warn you get out a minipet?"],
                get = function()
                          return MiniPet.db.char.GiveTextWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveTextWarning = not MiniPet.db.char.GiveTextWarning;
                      end,
            },
            [L["toggleButtonWarning"]] = {
                type = "toggle",
                name = L["Button Warning"],
                desc = L["Flash the button to warn you to get out a minipet?"],
                get = function()
                          return MiniPet.db.char.GiveButtonWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveButtonWarning = not MiniPet.db.char.GiveButtonWarning;
                      end,
            },
            [L["toggleMountWarning"]] = {
                type = "toggle",
                name = L["Mount Warning"],
                desc = L["Show text warnings to warn you get out a minipet when getting of a flying mount?"],
                get = function()
                          return MiniPet.db.char.GiveMountWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveMountWarning = not MiniPet.db.char.GiveMountWarning;
                      end,
            },
            [L["toggleBG"]] = {
                type = "toggle",
                name = L["Battle Ground Check"],
                desc = L["Show warnings in a battleground?"],
                get = function()
                          return MiniPet.db.char.BattleGroundCheck;
                      end,
                set = function( val )
                          MiniPet.db.char.BattleGroundCheck = not MiniPet.db.char.BattleGroundCheck;
                      end,
            },
            [L["lockButton"]] = {
                type = "toggle",
                name = L["Lock Button Position"],
                desc = L["Lock the position of the button?"],
                get = function()
                          return MiniPet.db.char.LockButton;
                      end,
                set = function( val )
                          MiniPet.db.char.LockButton = not MiniPet.db.char.LockButton;
                      end,
            },
            [L["list"]] = {
                type = "execute",
                name = L["List all MiniPets"],
                desc = L["Show a list with all your minipets."],
                func = "ListPets",
            },
            [L["relist"]] = {
                type = "execute",
                name = L["Refresh MiniPet list"],
                desc = L["Force a refresh of the list of all your minipets."],
                func = "GetPlayersPets",
            },
        },
    };

MiniPet = AceLibrary( "AceAddon-2.0" ):new( "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0" );
MiniPet.version = tonumber( string.match( "$Rev: 75305 $", "(%d+)" ) );
MiniPet:RegisterChatCommand( "/MiniPet", "/mp", options );
MiniPet:RegisterDB( "MiniPetDB", "MiniPetDBPC" );
MiniPet:RegisterDefaults( "char", {
        AutoHideButton = true,
        BattleGroundCheck = true,
        GiveTextWarning = true,
        GiveButtonWarning = true,
        GiveMountWarning = true,
        LockButton = false,
    } );
    
local PetAndMountDatabase = AceLibrary( "PetAndMountDatabase-1.0" );
local LibHelper = AceLibrary( "LibHelper-1.0" );
        
--Global Values
MiniPet.DEBUG               = false; --true = Debugging Mode
MiniPet.UpdateInterval      = 0.2;   --Time between two flashes of the button in seconds
MiniPet.TotalFlashTime      = 10;    --Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet.PlayersPets         = {};    --List of the players pet
MiniPet.CurrentPet          = nil;   --Pet on the Button
MiniPet.BoardedTaxi         = false;
MiniPet.StartedHearthstone  = false;
MiniPet.HearthstoneComplete = false;
MiniPet.PlayerDied          = false;
MiniPet.PlayerEnteringWorld = true;
MiniPet.ReadyToInitialize   = false;
MiniPet.PetListOkay         = false;

MiniPet.ShowButtonImg       = false; --Goes on and off if button flashes
MiniPet.FlashTime           = 0;     --Time since button flashes
MiniPet.FlashButton         = false; --On when the button flashes

local MINIPET_TELEPORTS_SPELLID = {
        8690,  --Hearthstone
        39937, --There's No Place Like Home (Ruby Slippers)
        --A Mage Teleports
        3565,  --Teleport: Darnassus
        32271, --Teleport: Exodar
        3562,  --Teleport: Ironforge
        33690, --Teleport: Shattrath
        3561,  --Teleport: Stormwind
        49359, --Teleport: Theramore
        --H Teleports
        3567,  --Teleport: Orgrimmar
        35715, --Teleport: Shattrath
        32272, --Teleport: Silvermoon
        49358, --Teleport: Stonard
        3566,  --Teleport: Thunder Bluff
        3563,  --Teleport: Undercity
        --Druid Teleport
        18960, --Teleport: Moonglade
        --Engineering Items
        36890, --Area52 Transporter
        23452, --Gnomish Transporter
        23442, --Everlook Transporter
        36941, --Toshley's Station Transporter
    };

local MINIPET_BATTLEGROUNDS = {
    	LBZ["Warsong Gulch"],
    	LBZ["Arathi Basin"],
    	LBZ["Alterac Valley"],
    	LBZ["Eye of the Storm"],
	};

function MiniPet:OnEnable()
    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function( bagID )
            if( MiniPet:HasMiniPetListChanged() )
                then MiniPet:ChooseRandomPet();
            end--if
        end );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD",
        function()
            MiniPet.PlayerDied = UnitIsDeadOrGhost( "player" );
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            MiniPet.PlayerEnteringWorld = false;
            MiniPet:WarnPlayer();
        end );
    self:RegisterEvent( "TAXIMAP_CLOSED",
        function()
            MiniPet.BoardedTaxi = true;
        end );
    self:RegisterEvent( "PLAYER_CONTROL_GAINED",
        function()
            if( MiniPet.BoardedTaxi ) then
                MiniPet.BoardedTaxi = false;
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_DEAD",
        function()
            LibHelper.dprint( MiniPet, "MiniPet: PlayerDied" );
            MiniPet.PlayerDied = true;
        end );
    self:RegisterEvent( "PLAYER_ALIVE",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            if( MiniPet.HearthstoneComplete ) then
                MiniPet.HearthstoneComplete = false;
                LibHelper.dprint( MiniPet, "MiniPet: Activating Pet After Hearthstone!" );
                MiniPet:WarnPlayer();
            end--if
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                LibHelper.dprint( MiniPet, "MiniPet: no longer dead!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_UNGHOST",
        function()
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                LibHelper.dprint( MiniPet, "MiniPet: no longer ghost!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED",
        function( unit, _, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    LibHelper.dprint( MiniPet, "MiniPet: Interrupted Teleporting!" );
                    MiniPet.StartedHearthstone = false;
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED",
        function( unit, spellName, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                     for _, spell in pairs( MINIPET_TELEPORTS_SPELLID ) do
                        if( spellName == GetSpellInfo( spell ) ) then
                            LibHelper.dprint( MiniPet, "MiniPet: Finished Teleporting!" );
                            MiniPet.StartedHearthstone = false;
                            MiniPet.HearthstoneComplete = true;
                            MiniPet:WarnPlayer();
                            break;
                        end--if
                    end--if
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SENT",
        function( unit, spellName, _, _ )
            if( unit == "player" ) then
                LibHelper.dprint( MiniPet, "Spellcast: " .. spellName );
                MiniPet.StartedHearthstone = false;
                for _, spell in pairs( MINIPET_TELEPORTS ) do
                    if( spellName == GetSpellInfo( spell ) ) then
                        LibHelper.dprint( MiniPet, "MiniPet: Started Teleporting!" );
                        MiniPet.StartedHearthstone = true;
                        break;
                    end--if
                end--for
            end--if
        end );
    self:RegisterEvent( "SpecialEvents_Dismounted",
        function( mountName, speed, flying )
            if( flying and MiniPet.db.char.GiveMountWarning ) then
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            MiniPet:SetButton( MiniPet.CurrentPet );
        end );

    --Upgrade check
    if( self.db.char.LastVersion == nil ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.char.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.char.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.char.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.char.AutoHideButton = MiniPet_HideButton;
    end--if
    if( self.db.char.LastVersion ~= self.version ) then
        self.db.char.LastVersion = self.version;
    end--if

    self.cooldown = CreateFrame( "Cooldown", MiniPet_Button:GetName() .. "Cooldown", MiniPet_Button, "CooldownFrameTemplate" );
    self.cooldown:SetReverse( false );
    self.cooldown:SetAllPoints( MiniPet_Button );
    self.cooldown:Hide();

    --Create the Pet Inventory
    self:GetPlayersPets();
    --Set a Pet
    self:ChooseRandomPet();
end--MiniPet:OnEnable();


function MiniPet:OnUpdate( elapsed )
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.ReadyToInitialize ) then
        LibHelper.dprint( self, "MiniPet: PlayerEnteringWorld and ALIVE" );
        self.PlayerEnteringWorld = false;
        self.ReadyToInitialize = false;
        self:WarnPlayer();
    end

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if( self.FlashButton ) then
        if( MiniPet_Button.TimeSinceLastUpdate == nil ) then
            MiniPet_Button.TimeSinceLastUpdate = 0;
            self.FlashTime = 0;
        end--if
        MiniPet_Button.TimeSinceLastUpdate = MiniPet_Button.TimeSinceLastUpdate + elapsed;
        self.FlashTime = self.FlashTime + elapsed;
        if( self.FlashTime < self.TotalFlashTime) then
            if( MiniPet_Button.TimeSinceLastUpdate > self.UpdateInterval ) then
                if( self.ShowButtonImg and ( self.CurrentPet ~= nil ) ) then
                    local _, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( self.CurrentPet );

                    LibHelper.SetButtonImage( MiniPet_Button, sItemTexture );
                    self.ShowButtonImg = false;
                else
                    LibHelper.SetButtonImage( MiniPet_Button, "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );
                    self.ShowButtonImg = true;
                end--if
                MiniPet_Button.TimeSinceLastUpdate = 0;
            end--if
        else
            self.FlashButton = false;
            self:SetButton( self.CurrentPet );
            self.FlashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

----------------------------
--+Pet/Inventory Functions--
--Pick a random pet and set the button.
function MiniPet:ChooseRandomPet()
    local playerPetList = self.PlayersPets;
    local amountOfPets = #playerPetList;

    if( amountOfPets > 0 ) then
        --math.random(upper) generates integer numbers between 1 and upper.
        local randomPetIndex = math.random( amountOfPets );

        if( amountOfPets > 1 ) then --Lets make sure the player gets a different pet if he gots more then one, the quick and dirty way.
            while( playerPetList[randomPetIndex] == self.CurrentPet ) do
                randomPetIndex = math.random( amountOfPets );
            end--while
        end--if
        self.CurrentPet = playerPetList[randomPetIndex];
    else
        self.CurrentPet = nil;
    end
    self:SetButton( self.CurrentPet );
end--MiniPet:ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersPets()
    self.PlayersPets = {};

    local playerPetList = self.PlayersPets;
    local iterator, handle = PetAndMountDatabase:IterateSet( "MiniPets" );
    local pet = iterator( handle );

    while pet do
        if( GetItemCount( pet ) > 0 ) then
            table.insert( playerPetList, pet );
        end--if
        pet = iterator( handle );
    end--for
end--MiniPet:GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet:ListPets()
    local playerPetList = self.PlayersPets;

    self:Print( L["MiniPet, List of your pets:"] );
    for _, pet in pairs( playerPetList ) do
        local sPetName, _, _, _, _, _, _, _ = GetItemInfo( pet );

        self:Print( "-|cFF00FFFF" .. sPetName );
    end--for
end--MiniPet:listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet:HasMiniPetListChanged()
    local prePetCount = #self.PlayersPets;
    local prePetList =  {};

    LibHelper.CopyTable( prePetList, self.PlayersPets );

    self:GetPlayersPets();

    local afterPetCount = #self.PlayersPets;

    -- differences in sizes always means the list has changed
    if( ( prePetCount ~= afterPetCount ) or ( not self.PetListOkay ) ) then
        self.PetListOkay = true;
        LibHelper.dprint( self, "Number of minipets changed" );
        return true;
    end--if
    self.PetListOkay = true;
    return not LibHelper.CompareTables( prePetList, self.PlayersPets );
end--MiniPet:HasMiniPetListChanged()

--Check if a player is in a battleground
function MiniPet:IsInBattleground()
    for _, bg in pairs( MINIPET_BATTLEGROUNDS ) do
        if( GetZoneText() == bg ) then
            return true;
        end--if
    end--for
    return false;
end--MiniPet:IsInBattleground()
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet:WarnPlayer()
    local charDB = self.db.char;

    if( ( not ( self:IsInBattleground() and charDB.BattleGroundCheck ) ) and ( not self.PlayerDied ) and ( not self.BoardedTaxi ) ) then--BG + option check
        if( charDB.GiveButtonWarning == true ) then
            self.FlashButton = true;
        end--if
        if( charDB.GiveTextWarning ) then
            if( #self.PlayersPets > 0 ) then    --No use if player isn't cool and gots no pets!
                MiniPet:wprint( L["ALERT: Time to summon your minipet!"] );
            end--if
        end--if
        if( charDB.AutoHideButton and ( not MiniPet_Button:IsVisible() ) ) then
            MiniPet_Button:Show();
        end--if
    end--if
end--MiniPet:WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet:wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet:wprint( msg )
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet:SetGameTooltips()
    local pet = MiniPet_Button:GetAttribute( "item1" );

    MiniPet_Tooltip:SetOwner( MiniPet_Button, "RIGHT" );
    MiniPet_Tooltip:SetBackdropColor( 0.0, 0.0, 0.0, 1.0 );
    if( pet == nil ) then
        MiniPet_Tooltip:SetText( L["Get a minipet for some company!"] );
    else
        MiniPet_Tooltip:SetHyperlink( pet );
    end--if
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( pet )
    local cooldownInicator = self.cooldown;

    if( not UnitAffectingCombat( "player" ) ) then
        cooldownInicator:Hide();
        if( pet == nil ) then
            MiniPet_Button:SetAttribute( "type1", "item" );
            MiniPet_Button:SetAttribute( "item1", nil );

            LibHelper.SetButtonImage( MiniPet_Button, "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );

            LibHelper.dprint( self, "No pets" );
        else
            local sName, itemLink, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( pet );
            local _, _, itemString = itemLink:find( "^|c%x+|H(.+)|h%[.+%]" );
            local startTime, duration, enable = GetItemCooldown( itemLink );

            MiniPet_Button:SetAttribute( "type1", "item" );
            MiniPet_Button:SetAttribute( "item1", itemString );

            LibHelper.SetButtonImage( MiniPet_Button, sItemTexture );

            if( sName == nil ) then
                self:Print( "Something is wrong with pet " .. pet );
            else
                LibHelper.dprint( self, "Pet set: " .. sName );
            end--if

            if( enable ) then
                cooldownInicator:Show();
                cooldownInicator:SetCooldown( startTime, duration );
            else
                cooldownInicator:Hide();
            end--if
        end--if
    end--if
end--MiniPet:SetButton( pet )
---UI Functions--
-----------------
